@@ -43,26 +43,7 @@ impl ToTokens for RawString {
 #[proc_macro]
 pub fn build(stream: proc_macro::TokenStream) -> proc_macro::TokenStream {
     let build = parse_macro_input!(stream as BuildMacro);
-
     let tokens = RawString(build.into_tokens_string());
-    let workspace_windows_dir = gen::workspace_windows_dir();
-
-    let mut destination = workspace_windows_dir.clone();
-    destination.pop();
-    destination.push("target");
-    let destination = RawString(
-        destination
-            .to_str()
-            .expect("Invalid workspace target dir")
-            .to_string(),
-    );
-
-    let workspace_windows_dir = RawString(
-        workspace_windows_dir
-            .to_str()
-            .expect("Invalid workspace windows dir")
-            .to_string(),
-    );
 
     let tokens = quote! {
         {
@@ -89,6 +70,7 @@ pub fn build(stream: proc_macro::TokenStream) -> proc_macro::TokenStream {
                             if file_type.is_file() {
                                 let path = file.path();
                                 if let ::std::option::Option::Some(filename) = path.file_name() {
+                                    let _ = std::fs::create_dir_all(&destination);
                                     destination.push(filename);
                                     let _ = ::std::fs::copy(path, &destination);
                                     destination.pop();
@@ -118,23 +100,45 @@ pub fn build(stream: proc_macro::TokenStream) -> proc_macro::TokenStream {
                 }
             }
 
-            if ::std::path::PathBuf::from(#workspace_windows_dir).exists() {
-                println!("cargo:rerun-if-changed={}", #workspace_windows_dir);
-                let mut source = ::std::path::PathBuf::from(#workspace_windows_dir);
+            let mut source : ::std::path::PathBuf = ::std::env::var("CARGO_MANIFEST_DIR").expect("No `CARGO_MANIFEST_DIR` env variable set").into();
+            source.push(".windows");
 
-                // The `target_arch` cfg is not set for build scripts so we need to sniff it out from the environment variable.
-                source.push(match ::std::env::var("CARGO_CFG_TARGET_ARCH").expect("No `CARGO_CFG_TARGET_ARCH` env variable set").as_str() {
-                    "x86_64" => "x64",
-                    "x86" => "x86",
-                    "arm" => "arm",
-                    "aarch64" => "arm64",
-                    unexpected => panic!("Unexpected `{}` architecture set by `CARGO_CFG_TARGET_ARCH`", unexpected),
-                });
+            println!("cargo:rerun-if-changed={}", source.to_str().expect("`CARGO_MANIFEST_DIR` not a valid path"));
 
-                let destination = ::std::path::PathBuf::from(#destination);
-                let profile = ::std::env::var("PROFILE").expect("No `PROFILE` env variable set");
-                copy_to_profile(&source, &destination, &profile);
+            // The `target_arch` cfg is not set for build scripts so we need to sniff it out from the environment variable.
+            source.push(match ::std::env::var("CARGO_CFG_TARGET_ARCH").expect("No `CARGO_CFG_TARGET_ARCH` env variable set").as_str() {
+                "x86_64" => "x64",
+                "x86" => "x86",
+                "arm" => "arm",
+                "aarch64" => "arm64",
+                unexpected => panic!("Unexpected `{}` architecture set by `CARGO_CFG_TARGET_ARCH`", unexpected),
+            });
+
+            println!("cargo:rustc-link-search=native={}", source.to_str().expect("`CARGO_MANIFEST_DIR` not a valid path"));
+            let mut destination : ::std::path::PathBuf = ::std::env::var("OUT_DIR").expect("No `OUT_DIR` env variable set").into();
+
+            loop {
+                destination.pop();
+                destination.push("Cargo.toml");
+
+                if destination.exists() {
+                    break;
+                }
+
+                destination.pop();
             }
+
+            destination.pop();
+            destination.push("target");
+
+            let profile = ::std::env::var("PROFILE").expect("No `PROFILE` env variable set");
+            copy_to_profile(&source, &destination, &profile);
+
+            destination.push(".windows");
+            destination.push("winmd");
+            source.pop();
+            source.push("winmd");
+            copy(&source, &mut destination);
         }
     };
 
